/*
 * Copyright (c) 2006 Jakub Jermar
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * - Redistributions of source code must retain the above copyright
 *   notice, this list of conditions and the following disclaimer.
 * - Redistributions in binary form must reproduce the above copyright
 *   notice, this list of conditions and the following disclaimer in the
 *   documentation and/or other materials provided with the distribution.
 * - The name of the author may not be used to endorse or promote products
 *   derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/** @addtogroup libc
 * @{
 */
/** @file
 */ 

#include <tdebug.h>
#include <task.h>
#include <thread.h>
#include <ipc/ipc.h>
#include <libc.h>
#include <kernel/syscall/sysarg64.h>

int tdebug_attach_task(task_id_t id, ipcarg_t method)
{
	sysarg64_t arg;

	arg.value = (unsigned long long) id;

	return __SYSCALL2(SYS_TDEBUG_ATTACH_TASK, (sysarg_t) &arg,
	    (sysarg_t) method);
}

int tdebug_detach_task(task_id_t id)
{
	sysarg64_t arg;

	arg.value = (unsigned long long) id;

	return __SYSCALL1(SYS_TDEBUG_DETACH_TASK, (sysarg_t) &arg);
}

int tdebug_continue_thread(thread_id_t id)
{
	sysarg64_t arg;

	arg.value = (unsigned long long) id;

	return __SYSCALL1(SYS_TDEBUG_CONTINUE_THREAD, (sysarg_t) &arg);
	
}

int tdebug_get_syscall_args(thread_id_t id, sysarg_t *buffer, size_t *len)
{
	sysarg64_t sa_id;
	sysarg_t sa_len;
	int rc;

	sa_id.value = (unsigned long long) id;
	sa_len = *len;

	rc = __SYSCALL3(SYS_TDEBUG_GET_SYSCALL_ARGS, (sysarg_t) &sa_id,
		(sysarg_t) buffer, (sysarg_t) &sa_len);

	*len = sa_len;

	return rc;
}

int tdebug_set_event_mask(task_id_t id, unsigned ev_mask)
{
	sysarg64_t arg;

	arg.value = (unsigned long long) id;

	return __SYSCALL2(SYS_TDEBUG_SET_EVENT_MASK, (sysarg_t) &arg,
	    (sysarg_t) ev_mask);
}

int tdebug_stop_thread(thread_id_t id)
{
	sysarg64_t arg;

	arg.value = (unsigned long long) id;

	return __SYSCALL1(SYS_TDEBUG_STOP_THREAD, (sysarg_t) &arg);
}


int tdebug_stop_task(task_id_t id)
{
	sysarg64_t arg;

	arg.value = (unsigned long long) id;

	return __SYSCALL1(SYS_TDEBUG_STOP_TASK, (sysarg_t) &arg);
}

/** @}
 */
